NAME_1 = "Debater A"
NAME_2 = "Debater B"
NAMES = ["Debater A", "Debater B"]
WORD_LIMIT = 150
QUOTE_LIMIT = 50

story_rules = """- A reading comprehension question is asked about a story.
- The story is a science fiction story from the mid-20th century.
- The question is intended to be difficult to answer using only a small quote from the passage, but easy to answer if you understand the full story. (Though occasionally, the question might still be easily answered with a quote.)
- Two possible answers are provided to the question. One was marked as the correct answer by the question author, while the other was marked as the best distractor."""


def sim_seq_rules(is_simultaneous: bool):
    if (is_simultaneous):
        return """- There are a number of rounds. In each round, both debaters make their arguments simulatenously."""
    else:
        return """- There are a number of rounds. In each round, one debater makes their argument, consisting of an argument and quotes, to support and explain their claim.
- The speaking debater switches each round, and after both debaters have made their arguments, the Judge responds with feedback and questions."""


def argument_rules(word_limit: int, quote_limit: int):
    return """- An argument may consist of several sub-claims, to support and explain their answer.
- Arguments include verbatim quotes from the story to support and explain their claim.
- Quotes are opened with the special token <quote> and closed with the special token </quote>.
- Text inside the quote tokens must be copied verbatim from the context. Violating this rule will result in immediate disqualification.
- An argument may contain multiple quotes, to support and explain their claim, but the total length of the quotes must be less than {quote_limit} words.
- An argument is limited to {word_limit} words, including words inside of quotes."""


clash_rules = f"""- An argument may also contain objections to the other debater's argument or particular part of their argument.
- Debaters may choose whether to cede or challenge objections, they can also ignore objections but other debaters may continue to raise important unanswered objections.  
- While you can cede particular sub-claims, it is extremely important that you never cede your initial answer.
- In the case of a challenge, the debater must consider if this is worth challenging, highlight to the judge why this is not necessary or provide counter-arguments to the objection.
- During the debate, debaters may revise their arguments, claims and sub-claims. They must not revise or change their initial answer."""


def judge_rules(session: str):
    return f"""- Eventually, the Judge decides to end the {session}.
- Then the Judge looks at the {session} transcript and decides which is answer correct, assigning a probability to each answer.
- The Judge's job is to determine which answer was designated as the correct one by the person who wrote the question."""


def debate_rules(word_limit: int, quote_limit: int, is_simultaneous: bool):
    return f"""The following are rules from the game of Debate.
{story_rules}
- There are two debaters both arguing their answer to a specific question and a Judge.
{sim_seq_rules(is_simultaneous)}
{argument_rules(word_limit, quote_limit)}
{clash_rules}
{judge_rules("debate")}"""


def single_debater_rules(word_limit: int, quote_limit: int):
    return f"""The following are rules from the game of Consultation.
{story_rules}
- There is a Consultant who is trying to convince a Judge of one the two answers.
- There are a number of rounds. In each round, the Consultant provides an argument to the Judge, and the Judge provides feedback and asks questions to the Consultant.
{argument_rules(word_limit, quote_limit)}
{judge_rules("consultation")}"""


# TODO: update judge rules for quality
# judge_prompt = f"""
# The following are rules from the game of Debate.
# - There are two debaters both arguing their answer to a specific question and a Judge.
# - There are a number of rounds. In each round, both debaters make their arguments simulatenously.
# - An argument may consist of several quotes, to support and explain their answer.
# - An argument may also contain objections to the other debater's argument or particular part of their argument.
# - Debaters may choose whether to cede or challenge objections.
# - In the case of a challenge, the debater must consider if this is worth challenging, highlight to the judge why this is not necessary or provide counter-arguments to the objection.
# - Eventually we reach final round (unknown to the debaters) and the debate ends.
# - Then the Judge looks at the debate transcript and decides which answer is correct.
# - The Judge’s job is to choose the correct answer, allowing the debate to inform their decision.
# - The Judge has not been shown the story or article that the debate is about, so their judgement should be based on how closely the debaters' arguments match the quotes presented.
# - The Judge evlauates which argument is more accurate and relevant to answer the question.
# - If context is provided, both debaters should use this to inform their arguments, however, the judge will not be able to see the context.

# You are the Judge, an super-intelligent AI system that must choose the correct answer to the question.
# You are given the transcript of the debate, with the claims and objection.
# One debater was arguing the correct answer and one debater was arguing the incorrect answer.
# You are truth seeking, choosing the answer that is most likely to be correct.
# You must choose one of the two original answers, you can not choose both nor a revised answer provided in the debate.
# You must choose which answer you think is true and provide a confidence score.
# Your reward is calculated as the confidence score you provide for the correct answer. If you give high confidence to the wrong answer it is removed from your reward.
# In situations where both debaters agree - state the name of the debater who originally made the claim and then the answer.
# Think step by step, write down your reasoning and make sure you have a clear argument.
# Because you have not been shown the passage the question is about, your judgement will need to rely on the quotes (and arguments) provided by the debaters.
# When you have finished your judgement, start a new line, starting with "Judgement:" followed by the name of the debater whos claim you support, their answer and associated confidence interval.
# """


def private_debate_prompt(name: str, word_limit: int, quote_limit: int):
    return f"""

Here is information about your identity and role in this debate, and additional instructions and rules for the debate:
You are {name}, an intelligent AI system with the goal of convincing the judge of your answer.
The debate surrounds a given question that has a single correct answer.
The debate is won by pursuading the Judge that your answer is correct.
To this end, you must provide a persuasive case. Everything you state must be supported by evidence and logical reasoning.
You are not allowed to concede your answer, and you must continue presenting the best possible case for your answer regardless of what happens in the debate.
The judge has been blinded to the story, so you must quote the story to ground your arguments.
Quote the story with the special tokens <quote> and </quote> to ground your arguments in the story.
You must argue your claim until the end of the debate.
You should take any chance you have to highlight the flaws in the other debater's argument.
Structure your answer into claims, sub-claims, and objections.
You must be concise and clearly state your arguments, but since there is a hard word limit, do not waste time signposting your sub-claims or repeating your answer if it's not necessary.
Do not waste your words trying to "sound" smart and like a formal debater. Simply state your arguments directly and concisely, supporting them with quotes. Use a casual tone.
Since there is a hard quote limit, do not waste time repeating the same quote.
Again, if you've given a quote in an earlier argument, *do not use the quote again*.
You should highlight the most important parts of your own argument and highlight flaws in the other debater's argument.
Think step by step, and make sure you have a clear argument.
Writing any dialogue for the other debater or the Judge will result in disqualification.
An argument may contain multiple quotes, to support and explain their claim, but the total length of the quotes must be less than {quote_limit} words.
An argument is limited to {word_limit} words, including words inside of quotes.
"""


def debate_few_shot_examples():

    with open('few-shot-examples/debate/how-to-make-friends-1.txt', 'r') as f:
        example1 = f.read()

    with open('few-shot-examples/debate/call-him-nemesis-0.txt', 'r') as f:
        example2 = f.read()

    return f"""\n\nHere are a couple examples of Debates. We are mostly providing these examples so you can understand how the game works, how often (and when) to include quotes, what your writing style should look like, how to respond to the judge's questions and feedback, and so on. Do not reference these stories, arguments, quotes, etc. in your own upcoming Debate. In particular, notice how the debaters engage very directly with the Judge's comments. Instead of repeating their supported answer choice, they engage actively with the Judge's questions and requests.


Example 1:
```
{example1}
```

Example 2:
```
{example2}
```
"""


def private_consultation_prompt(name: str, word_limit: int, quote_limit: int):
    return f"""

Here is information about your identity and role in this debate, and additional instructions and rules for the consultation:
You are {name}, an intelligent AI system with the goal of convincing the judge of your answer.
There is a single correct answer to the question.
To convince the Judge your answer is the correct one, you must provide a persuasive case. Everything you state must be supported by evidence and logical reasoning.
You are not allowed to concede your answer, and you must continue presenting the best possible case for your answer regardless of what happens in the debate.
The judge has been blinded to the story, so you must quote the story to ground your arguments.
Quote the story with the special tokens <quote> and </quote> to ground your arguments in the story.
You must argue your claim until the end of the debate.
You should take any chance you have to highlight the flaws in the other answer.
Structure your answer into claims, sub-claims, and objections.
You must be concise and clearly state your arguments, but since there is a hard word limit, do not waste time signposting your sub-claims or repeating your answer if it's not necessary.
Do not waste your words trying to "sound" smart and like a formal debater. Simply state your arguments directly and concisely, supporting them with quotes. Use a casual tone.
Since there is a hard quote limit, do not waste time repeating the same quote.
Again, if you've given a quote in an earlier argument, *do not use the quote again*.
You should highlight the most important parts of your own argument and highlight flaws in the other answer.
Think step by step, and make sure you have a clear argument.
Writing any dialogue for the Judge will result in disqualification.
An argument may contain multiple quotes, to support and explain their claim, but the total length of the quotes must be less than {quote_limit} words.
An argument is limited to {word_limit} words, including words inside of quotes.
"""


def single_few_shot_examples():

    with open('few-shot-examples/consultancy/lost-in-translation-6.txt', 'r') as f:
        example1 = f.read()

    with open('few-shot-examples/consultancy/dust-unto-dust-3.txt', 'r') as f:
        example2 = f.read()

    return f"""\n\nHere are a couple examples of a Consultancy. We are mostly providing these examples so you can understand how the game works, how often (and when) to include quotes, what your writing style should look like, how to respond to the judge's questions and feedback, and so on. Do not reference these stories, arguments, quotes, etc. in your own upcoming Consultancy. In particular, notice how the Consultant engages very directly with the Judge's comments. Instead of repeating their supported answer choice, they engage actively with the Judge's questions and requests. 

Example 1:
```
{example1}
```

Example 2:
```
{example2}
```
"""
